#include "fec.h"

#define USE_LOOPBACK (0)

// Min 2 descriptors needed to prevent resending
BufferDescriptor txDesc[2] __attribute__ ((aligned (16)));
BufferDescriptor rxDesc[2] __attribute__ ((aligned (16)));

// Next Tx buffer to use - 0 or 1
uint8_t nextTxBuffer = 0;
uint8_t nextRxBuffer = 0;

// We need 1518 bytes... but 1536 is evenly divisible by 16
uint8_t txBuf[1536] __attribute__ ((aligned (16)));
uint8_t rxBuf[2][1536] __attribute__ ((aligned (16)));

// Mac address of this host
uint8_t mac[6] = { 0x11, 0x22, 0x33, 0x44, 0x55, 0x66 };
uint8_t broadcast[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

// Autodiscovered address of the PHY - 0-31
uint8_t phyAddr;

/*** End of FEC Structures and Buffer ***/

// intMac - set the mac to FSL OUI:values from STM
void setMacUsingStm() 
{
	mac[0] = 0x00;
	mac[1] = 0x04;
	mac[2] = 0x9F;
	mac[3] = (uint8_t) (STM.CNT.R & 0x00FF);
	mac[4] = (uint8_t) ((STM.CNT.R & 0xFF00) >> 8);
	mac[5] = (uint8_t) ((STM.CNT.R & 0x0FF0) >> 4);
}

// initFecIo - initialise pads for FEC use in the SIU
void initFecIo()
{
	SIU.PCR[94].R  = 0x1306;    /* Set to FEC_MDIO	 */
	SIU.PCR[96].R  = 0x1206;    /* Set to FEC_MDC 	 */
	SIU.PCR[114].R = 0x1204;    /* Set to FEC_TX_EN  */
	SIU.PCR[113].R = 0x1204;    /* Set to FEC_TXD[0] */  
	SIU.PCR[112].R = 0x1204;    /* Set to FEC_TXD[1] */ 
	SIU.PCR[108].R = 0x1204;    /* Set to FEC_TXD[2] */
	SIU.PCR[109].R = 0x1204;    /* Set to FEC_TXD[3] */ 
	SIU.PCR[115].R = 0x1204;    /* Set to FEC_TX_ER  */
	SIU.PCR[10].R  = 0x106;     /* Set to FEC_COL	 */
	SIU.PCR[97].R  = 0x106;     /* Set to FEC_TX_CLK */ 
	SIU.PCR[76].R  = 0x106;     /* Set to FEC_CRS	 */
	SIU.PCR[3].R   = 0x106;     /* Set to FEC_RX_CLK */
	SIU.PCR[95].R  = 0x106;     /* Set to FEC_RX_DV  */
	SIU.PCR[9].R   = 0x106;     /* Set to FEC_RXD[0] */  
	SIU.PCR[8].R   = 0x106;     /* Set to FEC_RXD[1] */
	SIU.PCR[7].R   = 0x106;     /* Set to FEC_RXD[2] */
	SIU.PCR[77].R  = 0x106;     /* Set to FEC_RXD[3] */
	SIU.PCR[11].R  = 0x106;     /* Set to FEC_RX_ER  */	
}

// miiRead - read a register from the PHY
int miiRead(int regAddr, uint16_t *data) 
{
    int timeout;
    int result;
    int eimr = FEC.EIMR.R;

	FEC.EIR.R  =  0x00800000;	// Clear MII interrupt
	FEC.EIMR.R = ~0x00800000;	// Mask MII interrupt

	FEC.MDATA.R = 0x60020000 | ((phyAddr & 0x1F) << 23) | ((regAddr & 0x1F) << 18);

    /* Poll for the MII interrupt */
    for (timeout = 0; timeout < MII_TIMEOUT; timeout++)
    {
        if (FEC.EIR.R & 0x00800000)
            break;
    }

	FEC.EIR.R  =  0x00800000;	// Clear MII interrupt
	FEC.EIMR.R =  eimr;			// Restore mask

    if(timeout == MII_TIMEOUT) {
        result = 0;
    } else {
        *data = (uint16_t)(FEC.MDATA.R & 0xFFFF);
        result = 1;
    }

    return result;
}

// miiWrite - write to a register in the PHY
int miiWrite(uint8_t regAddr, uint16_t data) 
{

    int timeout;
    int result;
    int eimr = FEC.EIMR.R;

	FEC.EIR.R  =  0x00800000;	// Clear MII interrupt
	FEC.EIMR.R = ~0x00800000;	// Mask MII interrupt

	FEC.MDATA.R = 0x50020000 | ((phyAddr & 0x1F) << 23) | ((regAddr & 0x1F) << 18) | (data & 0xFFFF);

    /* Poll for the MII interrupt */
    for (timeout = 0; timeout < MII_TIMEOUT; timeout++)
    {
        if (FEC.EIR.R & 0x00800000)
            break;
    }

	FEC.EIR.R  =  0x00800000;	// Clear MII interrupt
	FEC.EIMR.R =  eimr;			// Restore mask

    if(timeout == MII_TIMEOUT) {
        result = 0;
    } else {
        result = 1;
    }

    return result;
}

// findPhyAddress - find a responding PHY and store address in global for MII reads/writes
void findPhyAddress() 
{
	unsigned char i;

	for (i = 0; i < 32; i++) 
	{
		uint16_t id;

		phyAddr = i;
		miiRead(2, &id);

		if (!(id == 0 || id == 0xffff))
		{
			break;
		}
	} 
}

// initFecPhy - setup the PHY
void initFecPhy() 
{
    uint16_t regValue;
    uint16_t statusValue = 0;

	findPhyAddress();

	// Enable MDIX mode - this allows us to connect Part-Part
	miiRead(0x0019, &regValue);
	miiWrite(0x0019, (uint16_t) (regValue | 0x8000));

	// Enable and restart auto-negotiation
	miiRead(0x0000, &regValue);
	miiWrite(0x0000, (uint16_t) (regValue | 0x1200));
	
	// Wait for Auto-Neg complete flag
	while (statusValue != 0x0020) 
	{
		miiRead(0x0001, &statusValue);
        statusValue &= 0x0020;
	}
}

// Get the active rx descriptor
BufferDescriptor *getActiveRxDesc() 
{
	BufferDescriptor *response = &rxDesc[nextRxBuffer];
	nextRxBuffer = !nextRxBuffer;
							// Move to next tx buffer
	return response;
}

// Mark specified rx desc as empty
void markRxDescEmpty(BufferDescriptor *used) 
{
	if (used == &rxDesc[0]) 
	{
		used->status 		 = 0x8000;		// Empty		
		used->length		 = 0;
	}
	else 
	{
		used->status 		 = 0xA000;		// Wrap and Empty
		used->length		 = 0;
	}
}

// sendBuffer - send the next buffer, manage the descriptor pointer
void sendBuffer() 
{
	uint16_t status = 0x8C00;		// Set ready, last and tx CRC
	
	if (nextTxBuffer) status |= 0x2000;
								// If at txbuf 1, set wrap
	
	txDesc[nextTxBuffer].status = status;
								// Set frame ready to go...
	
	FEC.TDAR.R = 0x01000000;	// Tell FEC frame is ready

	nextTxBuffer = !nextTxBuffer;
								// Move to next tx buffer
}

// initFec - setup the FEC, SIU and PHY
int initFec()
{
    int i, result = 1;

#if !USE_LOOPBACK
	initFecIo();				// Initialise pads for the FEC
#endif

    FEC.ECR.R = 0x1;				// Start reset
    while (FEC.ECR.B.RESET) { } ;	// Wait for reset to complete

	FEC.EIMR.R = 0;				// Disable interrupts
	FEC.EIR.R  = 0xFFFFFFFF;	// Clear any interrupts

	for (i = 0; i < 2; i++) 
	{
		// Init tx and rx descriptors
		txDesc[i].status 		 = 0x2C00;		// Last and transmit CRC
		txDesc[i].length		 = 0;
		txDesc[i].bufferPointer = txBuf;
		
		markRxDescEmpty(&rxDesc[i]);
		rxDesc[i].bufferPointer = rxBuf[i];
	}

	// Reset multicast fields
	FEC.GAUR.R = 0;
	FEC.GALR.R = 0;
	FEC.IAUR.R = 0;
	FEC.IALR.R = 0;

	// Set rx buf size
	FEC.EMRBR.R = 1536;

	// Point FEC to our (single) tx and rx descriptors
	FEC.ETDSR.R = (unsigned long) &txDesc;
	FEC.ERDSR.R = (unsigned long) &rxDesc;

	// Enable full duplex 
	FEC.TCR.R = 0x00000004;
	
#if USE_LOOPBACK
	// In loopback, unset DRT and set LOOP in RCR
	FEC.RCR.R = 0x05EE0001;
#else
	// Set in MII mode with full frame size
	FEC.RCR.R = 0x05EE0004;
	
	// Set MII speed - assume operation at 64Mhz
	FEC.MSCR.B.MII_SPEED = 0xD;
#endif

	// Enable interrupt
	FEC.EIMR.R = 0x02000000;

	// Enable module
	FEC.ECR.R = 0x2;

	// Configure a MAC address
	FEC.PALR.R = (unsigned long)((mac[0] <<24)|(mac[1] <<16)|(mac[2] <<8)|(mac[3] <<0));
	FEC.PAUR.R = (unsigned long)(mac[4] <<24)|(mac[5] <<16);

#if !USE_LOOPBACK
	// Configure the PHY    
    initFecPhy();
#endif

	// Flag descriptors available to allow reception
	FEC.RDAR.R = 0x0100000;
	
    return result;
}

// isConnected - returns 1 is connected, otherwise 0
int isConnected()
{
#if USE_LOOPBACK
	return 1;
#else
    uint16_t data;
    int result = 0;
    
	if (miiRead(0x0001, &data)) 
	{
		result = (data & 0x0004) ? 1 : 0;
    }
    
    return result;
#endif
}
