#include "MPC5646C.h"

#include "fnet.h"
#include "fnet_user_config.h"

/* Prototype for PRC1 startup */
#ifdef __cplusplus
  extern "C"{void __start_p1();}
#else
  extern void __start_p1();
#endif

#define PORT_NUMBER  (1080)

struct command_message 
{
	int portNumber;
	int command;
};

int main(void) {
	char message[] = "OK";
	SOCKET main_socket = 0;
	struct sockaddr_in local_info;
	struct sockaddr_in remote_info;
	int remote_info_len = sizeof(struct sockaddr_in);
	
	struct command_message command;
	
    SWT.SR.R = 0xC520;          /* Clear Soft lock bit in CR by writing     */
    SWT.SR.R = 0xD928;          /*  0xC520 followed by 0xD928 to WSC field  */
    SWT.CR.B.WEN = 0x0;         /* Clear Watchdog Enable Bit -> 0xC000010A  */

    ME.MER.R = 0x000025FF;          /* Enable all modes                     */		
    ME.RUNPC[0].R = 0x000000FE;     /* Enable all peripherals in all modes  */
    
    ECSM.MUDCR.R=0x40000000; /* add a wait state since Fsys > 100 Mhz*/

   /* Enable system clock for all peripherals, Clock divider of 1           */
    CGM.SC_DC0.R = 0x81;
    CGM.SC_DC1.R = 0x82;
    CGM.SC_DC2.R = 0x8F;

	CGM.FEC_DCR.R = 0x1;

   	CGM.AC0_SC.B.SELCTL = 0x1; /* selects PLL to be driven by FXOSC */

	/* these next 3 lines can be optimised in to a single write */
	ME.RUN[0].B.FXOSC0ON = 1;       /* Enable external osc */ 
	ME.RUN[0].B.FMPLLON  = 1;       /* Enable PLL */
	ME.RUN[0].B.SYSCLK   = 0x4;     /* System clock is PLL */

	/* these next 3 lines can be optimised in to a single write */
	CGM.FMPLL_CR.B.IDF  = 0x3;  /* Input Divider  = 4  -> 10 MHz   */
	CGM.FMPLL_CR.B.NDIV = 48;   /* Loop Divider   = 48 -> 480 MHz */
	CGM.FMPLL_CR.B.ODF  = 0x1;  /* Output Divider = 4  -> 120 MHz  */

	ME.MCTL.R = 0x40005AF0;         /* Mode & Key */
	ME.MCTL.R = 0x4000A50F;         /* Mode & Key inverted */
	while(ME.GS.B.S_MTRANS==1) {};  /* Wait for mode entry to complete */

	INTC_InitINTCInterrupts();
	INTC.CPR_PRC0.B.PRI = 0;
	
	if (fnet_init_static() != FNET_ERR) 
	{
		fnet_dhcp_init(fnet_netif_find_name("eth0"), 0);

		// Step 1 Create a socket
		main_socket = socket(AF_INET, SOCK_DGRAM, 0);

		// Step 2 Create a sockaddr_in to describe the local port
		local_info.sin_len = sizeof(local_info);
		local_info.sin_family = AF_INET;
		local_info.sin_addr.s_addr = fnet_htonl(INADDR_ANY);
		local_info.sin_port = fnet_htons(PORT_NUMBER);

		remote_info.sin_family = AF_INET;

		// Step 3 Bind the socket to the port
		int r = bind(main_socket, (struct sockaddr *) &local_info, sizeof(local_info));

		while(1) 
		{	
			// Step 4 Check for packet
			r = recvfrom(main_socket, (char *) &command, sizeof(command), 0, 
							(struct sockaddr *) &remote_info, &remote_info_len);
			
			// Step 5 If packet found, process
			if (r > 0) 
			{
				// Make port output and set to !value (1 = logic low out = On, 0 = logic high out = off)
				SIU.PCR[command.portNumber].R = 0x200;
				SIU.GPDO[command.portNumber].R = !command.command;
			
				r = sendto(main_socket, message, sizeof(message), 0,
			       (struct sockaddr *) &remote_info, sizeof(remote_info));
			}

			// Step 6 Process any FNET services (i.e. DHCP)
        	fnet_poll();
		}
	}
}



