#include "MPC5646C.h"

#include "fnet.h"
#include "fnet_user_config.h"

extern const struct fnet_fs_rom_image fnet_fs_image;

unsigned long writeCgiPage(char * buffer, unsigned long buffer_size, char * eof, long *cookie) 
{
	char *messageTemplate="[{batt:%d}]";

	*eof = 1;

	return sprintf(buffer, messageTemplate, ADC_0.CDR[0].B.CDATA);
}

unsigned long miniCgiPage(char * buffer, unsigned long buffer_size, char * eof, long *cookie) 
{
	char *miniTemplate="%d";

	*eof = 1;

	return sprintf(buffer, miniTemplate, 69);
}

void delay(int dvalue) 
{
	int delay;
	for (delay = 0; delay < dvalue; delay++);
}

int handlePostUrlReadingFromQueryString(char * query, long *cookie) 
{
	int output;
	
	sscanf(query, "%d", &output);
	
	SIU.PCR[output].R = 0x200;
	SIU.GPDO[output].R = 0;
	delay(0xFFFF);
	SIU.GPDO[output].R = 1;
	delay(0xFFFF);
	SIU.GPDO[output].R = 0;
	delay(0xFFFF);
	SIU.GPDO[output].R = 1;
	
	return FNET_OK;
}

unsigned long sendPostUrlReadingFromQueryString(char * buffer, unsigned long buffer_size, char * eof, long *cookie) 
{
	*eof = 1;
	buffer = "<html><body>OK</body></html>";
	return strlen(buffer);
}

int main(void) {
    SWT.SR.R = 0xC520;          /* Clear Soft lock bit in CR by writing     */
    SWT.SR.R = 0xD928;          /*  0xC520 followed by 0xD928 to WSC field  */
    SWT.CR.B.WEN = 0x0;         /* Clear Watchdog Enable Bit -> 0xC000010A  */

    ME.MER.R = 0x000025FF;          /* Enable all modes                     */		
    ME.RUNPC[0].R = 0x000000FE;     /* Enable all peripherals in all modes  */
    
    ECSM.MUDCR.R=0x40000000; /* add a wait state since Fsys > 100 Mhz*/

   /* Enable system clock for all peripherals, Clock divider of 1           */
    CGM.SC_DC0.R = 0x81;
    CGM.SC_DC1.R = 0x82;
    CGM.SC_DC2.R = 0x8F;

	CGM.FEC_DCR.R = 0x1;

   	CGM.AC0_SC.B.SELCTL = 0x1; /* selects PLL to be driven by FXOSC */

	/* these next 3 lines can be optimised in to a single write */
	ME.RUN[0].B.FXOSC0ON = 1;       /* Enable external osc */ 
	ME.RUN[0].B.FMPLLON  = 1;       /* Enable PLL */
	ME.RUN[0].B.SYSCLK   = 0x4;     /* System clock is PLL */

	/* these next 3 lines can be optimised in to a single write */
	CGM.FMPLL_CR.B.IDF  = 0x3;  /* Input Divider  = 4  -> 10 MHz   */
	CGM.FMPLL_CR.B.NDIV = 48;   /* Loop Divider   = 48 -> 480 MHz */
	CGM.FMPLL_CR.B.ODF  = 0x1;  /* Output Divider = 4  -> 120 MHz  */

	ME.MCTL.R = 0x40005AF0;         /* Mode & Key */
	ME.MCTL.R = 0x4000A50F;         /* Mode & Key inverted */
	while(ME.GS.B.S_MTRANS==1) {};  /* Wait for mode entry to complete */

	INTC_InitINTCInterrupts();
	INTC.CPR_PRC0.B.PRI = 0;
	
	if (fnet_init_static() != FNET_ERR) 
	{
		fnet_dhcp_init(fnet_netif_find_name("eth0"), 0);

	    SIU.PCR[20].R = 0x2000;	// PB4

	    ADC_0.MCR.B.PWDN = 0;		/* ADC enable */
	    ADC_0.MCR.B.MODE = 1; 	/* scan mode */
	    ADC_0.MCR.B.CTUEN = 0;	/* start by SW */
	    ADC_0.NCMR0.B.CH0 = 1;	/* mask enable for channel0 */
	    ADC_0.CIMR0.R = 1;	/* mask enable for channel0 */
	    ADC_0.MCR.B.NSTART = 1;	/* ADC Go! */

		// Initialise the filesystem
		if( fnet_fs_init( ) == FNET_OK)
	    {
	        /* Register the FNET ROM FS. */
	        fnet_fs_rom_register( );
	 
	        /* Mount the FNET ROM FS image. */
	        if( fnet_fs_mount( FNET_FS_ROM_NAME, FNET_FS_ROM_NAME, (void *)&fnet_fs_image ) == FNET_ERR )
	            fnet_printf("Failed to mount FNET FS");
	    }
	    else
	    {
	        fnet_printf("Failed to init FNET FS");
	    } 

		struct fnet_http_cgi cgihandler[2];
		
		cgihandler[0].name = "test.cgi";
		cgihandler[0].handle = 0;
		cgihandler[0].send = writeCgiPage;

		cgihandler[1].name = "mini.cgi";
		cgihandler[1].handle = 0;
		cgihandler[1].send = miniCgiPage;

		struct fnet_http_params params;

// Only enable POST handler if POST support is enabled
#if FNET_CFG_HTTP_POST
		struct fnet_http_post posthandler[1];
		
		posthandler[0].name = "post.php";
		posthandler[0].handle = handlePostUrlReadingFromQueryString;
		posthandler[0].receive = 0;
		posthandler[0].send = sendPostUrlReadingFromQueryString;

        params.post_table = posthandler;
#endif
        
        params.root_path = FNET_FS_ROM_NAME;    /* Root directory path */
        params.index_path = "index.html";   /* Index file path, relative to the root_path */
        params.port = 80;
        params.ip_address = INADDR_ANY;
        params.cgi_table = cgihandler;

		/* Enable HTTP server */
        fnet_http_desc_t http_desc = fnet_http_init(&params);
        
        fnet_printf("HTTP Inited");
        
        while(1) 
        {
        	fnet_poll();
        }
	}
}



